@@ -22,6 +22,7 @@
 #ifdef WITH_PROFILES
 #include "profilesmodel.h"
 #endif
+#include "primaryoutputcombo.h"
 
 #include <QVBoxLayout>
 #include <QSplitter>
@@ -38,6 +39,7 @@
 #include <kscreen/mode.h>
 #include <kscreen/config.h>
 #include <kscreen/getconfigoperation.h>
+#include <kscreen/configmonitor.h>
 
 #include <QtCore/QDir>
 #include <QStandardPaths>
@@ -63,10 +65,10 @@ Widget::Widget(QWidget *parent):
     QSplitter *splitter = new QSplitter(Qt::Vertical, this);
     layout->addWidget(splitter);
 
-    m_declarativeView = new QQuickView();
-    QWidget *container = QWidget::createWindowContainer(m_declarativeView, this);
-    m_declarativeView->setResizeMode(QQuickView::SizeRootObjectToView);
-    m_declarativeView->setMinimumHeight(280);
+    mDeclarativeView = new QQuickView();
+    QWidget *container = QWidget::createWindowContainer(mDeclarativeView, this);
+    mDeclarativeView->setResizeMode(QQuickView::SizeRootObjectToView);
+    mDeclarativeView->setMinimumHeight(280);
     container->setMinimumHeight(280);
     splitter->addWidget(container);
 
@@ -81,12 +83,9 @@ Widget::Widget(QWidget *parent):
     QHBoxLayout *hbox = new QHBoxLayout;
     vbox->addLayout(hbox);
 
-    mPrimaryCombo = new QComboBox(this);
-    mPrimaryCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);
-    mPrimaryCombo->addItem(i18n("No primary screen"));
-    connect(mPrimaryCombo, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
-            this, &Widget::slotPrimaryChanged);
-
+    mPrimaryCombo = new PrimaryOutputCombo(this);
+    connect(mPrimaryCombo, &PrimaryOutputCombo::changed,
+            this, &Widget::changed);
     hbox->addWidget(new QLabel(i18n("Primary display:")));
     hbox->addWidget(mPrimaryCombo);
 
@@ -104,12 +103,12 @@ Widget::Widget(QWidget *parent):
     hbox->addWidget(mProfilesCombo);
 #endif
 
-    m_controlPanel = new ControlPanel(this);
-    connect(m_controlPanel, &ControlPanel::changed,
+    mControlPanel = new ControlPanel(this);
+    connect(mControlPanel, &ControlPanel::changed,
             this, &Widget::changed);
-    vbox->addWidget(m_controlPanel);
+    vbox->addWidget(mControlPanel);
 
-    mUnifyButton = new QPushButton(i18n("Unify outputs"), this);
+    mUnifyButton = new QPushButton(i18n("Unify Outputs"), this);
     connect(mUnifyButton, &QPushButton::clicked,
             [&](bool clicked) {
                 Q_UNUSED(clicked);
@@ -150,33 +149,37 @@ bool Widget::eventFilter(QObject* object, QEvent* event)
 void Widget::setConfig(const KScreen::ConfigPtr &config)
 {
     if (mConfig) {
-        Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
-            disconnect(output.data(), &KScreen::Output::isConnectedChanged,
-                       this, &Widget::slotOutputConnectedChanged);
-            disconnect(output.data(), &KScreen::Output::isEnabledChanged,
-                       this, &Widget::slotOutputEnabledChanged);
-            disconnect(output.data(), &KScreen::Output::isPrimaryChanged,
-                       this, &Widget::slotOutputPrimaryChanged);
-            disconnect(output.data(), &KScreen::Output::posChanged,
-                       this, &Widget::changed);
+        KScreen::ConfigMonitor::instance()->removeConfig(mConfig);
+        for (const KScreen::OutputPtr &output : mConfig->outputs()) {
+            output->disconnect(this);
         }
     }
 
     mConfig = config;
+    KScreen::ConfigMonitor::instance()->addConfig(mConfig);
+
     mScreen->setConfig(mConfig);
-    m_controlPanel->setConfig(mConfig);
-    Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
-        connect(output.data(), &KScreen::Output::isConnectedChanged,
-                this, &Widget::slotOutputConnectedChanged);
+    mControlPanel->setConfig(mConfig);
+    mPrimaryCombo->setConfig(mConfig);
+
+    for (const KScreen::OutputPtr &output : mConfig->outputs()) {
         connect(output.data(), &KScreen::Output::isEnabledChanged,
                 this, &Widget::slotOutputEnabledChanged);
-        connect(output.data(), &KScreen::Output::isPrimaryChanged,
-                this, &Widget::slotOutputPrimaryChanged);
         connect(output.data(), &KScreen::Output::posChanged,
                 this, &Widget::changed);
     }
 
-    initPrimaryCombo();
+    // Select the primary (or only) output by default
+    QMLOutput *qmlOutput = mScreen->primaryOutput();
+    if (qmlOutput) {
+        mScreen->setActiveOutput(qmlOutput);
+    } else {
+        if (mScreen->outputs().count() > 0) {
+            mScreen->setActiveOutput(mScreen->outputs()[0]);
+        }
+    }
+
+    slotOutputEnabledChanged();
 }
 
 KScreen::ConfigPtr Widget::currentConfig() const
@@ -195,14 +198,14 @@ void Widget::loadQml()
 
     //const QString file = QDir::currentPath() + "/main.qml";
     const QString file = QStandardPaths::locate(QStandardPaths::QStandardPaths::GenericDataLocation, QStringLiteral("kcm_kscreen/qml/main.qml"));
-    m_declarativeView->setSource(QUrl::fromLocalFile(file));
+    mDeclarativeView->setSource(QUrl::fromLocalFile(file));
 
-    QQuickItem* rootObject = m_declarativeView->rootObject();
+    QQuickItem* rootObject = mDeclarativeView->rootObject();
     mScreen = rootObject->findChild<QMLScreen*>(QLatin1String("outputView"));
     if (!mScreen) {
         return;
     }
-    mScreen->setEngine(m_declarativeView->engine());
+    mScreen->setEngine(mDeclarativeView->engine());
 
     connect(mScreen, &QMLScreen::focusedOutputChanged,
             this, &Widget::slotFocusedOutputChanged);
@@ -210,68 +213,10 @@ void Widget::loadQml()
             this, SLOT(slotIdentifyButtonClicked()));
 }
 
-void Widget::initPrimaryCombo()
-{
-    mPrimaryCombo->blockSignals(true);
-    mPrimaryCombo->clear();
-    mPrimaryCombo->addItem(i18n("No primary output"));
-
-    Q_FOREACH (const KScreen::OutputPtr &output, mConfig->outputs()) {
-        if (!output->isConnected() || !output->isEnabled()) {
-            continue;
-        }
-
-        mPrimaryCombo->addItem(Utils::outputName(output), output->id());
-        if (output->isPrimary()) {
-            mPrimaryCombo->setCurrentIndex(mPrimaryCombo->count() - 1);
-        }
-    }
-    mPrimaryCombo->blockSignals(false);
-}
-
 
 void Widget::slotFocusedOutputChanged(QMLOutput *output)
 {
-    m_controlPanel->activateOutput(output->outputPtr());
-}
-
-void Widget::slotOutputPrimaryChanged()
-{
-    const int id = qobject_cast<KScreen::Output*>(sender())->id();
-    const int index = mPrimaryCombo->findData(id);
-    if (index == -1) { // No primary
-        mPrimaryCombo->setCurrentIndex(0);
-        return;
-    }
-    mPrimaryCombo->blockSignals(true);
-    mPrimaryCombo->setCurrentIndex(index);
-    mPrimaryCombo->blockSignals(false);
-}
-
-void Widget::slotPrimaryChanged(int index)
-{
-    const int id = mPrimaryCombo->itemData(index).toInt();
-    for (KScreen::OutputPtr &output : mConfig->outputs()) {
-        output->blockSignals(true);
-        output->setPrimary(output->id() == id);
-        output->blockSignals(false);
-    }
-
-    Q_EMIT changed();
-}
-
-void Widget::slotOutputConnectedChanged()
-{
-    KScreen::Output *output = qobject_cast<KScreen::Output*>(sender());
-    if (output->isConnected()) {
-        mPrimaryCombo->addItem(Utils::outputName(output), output->id());
-        if (output->isPrimary()) {
-            mPrimaryCombo->setCurrentIndex(mPrimaryCombo->count() - 1);
-        }
-    } else {
-        const int index = mPrimaryCombo->findData(output->id());
-        mPrimaryCombo->removeItem(index);
-    }
+    mControlPanel->activateOutput(output->outputPtr());
 }
 
 void Widget::slotOutputEnabledChanged()
@@ -287,14 +232,6 @@ void Widget::slotOutputEnabledChanged()
         }
     }
 
-    KScreen::Output *output = qobject_cast<KScreen::Output*>(sender());
-    if (output->isEnabled()) {
-        mPrimaryCombo->addItem(Utils::outputName(output), output->id());
-    } else {
-        const int index = mPrimaryCombo->findData(output->id());
-        mPrimaryCombo->removeItem(index);
-    }
-
     mUnifyButton->setEnabled(enabledOutputsCnt > 1);
 }
 
@@ -360,9 +297,9 @@ void Widget::slotUnifyOutputs()
         mScreen->updateOutputsPlacement();
 
         mPrimaryCombo->setEnabled(false);
-        m_controlPanel->setUnifiedOutput(base->outputPtr());
+        mControlPanel->setUnifiedOutput(base->outputPtr());
 
-        mUnifyButton->setText(i18n("Break unified outputs"));
+        mUnifyButton->setText(i18n("Break Unified Outputs"));
     }
 
     Q_EMIT changed();
